@@ -25,7 +25,9 @@
 #include <arpa/inet.h>
 #include "NetworkManagerSystem.h"
 #include "NetworkManagerUtils.h"
-#include "NetworkManagerDevice.h"
+#include "nm-device.h"
+#include "nm-device-802-3-ethernet.h"
+#include "nm-device-802-11-wireless.h"
 #include "NetworkManagerDialup.h"
 #include "nm-utils.h"
 #include "shvar.h"
@@ -319,7 +321,10 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 	struct ether_addr hw_addr;
 	unsigned char eui[8];
 
-	nm_device_get_hw_address(dev, &hw_addr);
+	if (nm_device_is_802_3_ethernet (dev))
+		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
+	else if (nm_device_is_802_11_wireless (dev))
+		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
 
 	memcpy (eui, &(hw_addr.ether_addr_octet), sizeof (hw_addr.ether_addr_octet));
 	memmove (eui+5, eui+3, 3);
@@ -340,6 +345,7 @@ typedef struct RHSystemConfigData
 {
 	NMIP4Config *	config;
 	gboolean		use_dhcp;
+	gboolean		system_disabled;
 } RHSystemConfigData;
 
 
@@ -492,6 +498,17 @@ void *nm_system_device_get_system_config (NMDevice *dev)
 		free (buf);
 	}
 
+	if ((buf = svGetValue (file, "NM_CONTROLLED")))
+	{
+		nm_debug ("NM_CONTROLLED=%s", buf);
+		if (!strcasecmp (buf, "no"))
+		{
+			nm_info ("System configuration disables device %s", nm_device_get_iface (dev));
+			sys_data->system_disabled = TRUE;
+		}
+		free (buf);
+	}		
+
 	sys_data->config = nm_ip4_config_new ();
 
 	if (!(sys_data->use_dhcp))
@@ -632,6 +649,26 @@ gboolean nm_system_device_get_use_dhcp (NMDevice *dev)
 }
 
 
+/*
+ * nm_system_device_get_disabled
+ *
+ * Return whether the distro-specific system config tells us to use
+ * dhcp for this device.
+ *
+ */
+gboolean nm_system_device_get_disabled (NMDevice *dev)
+{
+	RHSystemConfigData *sys_data;
+
+	g_return_val_if_fail (dev != NULL, FALSE);
+
+	if ((sys_data = nm_device_get_system_config_data (dev)))
+		return sys_data->system_disabled;
+
+	return FALSE;
+}
+
+
 NMIP4Config *nm_system_device_new_ip4_system_config (NMDevice *dev)
 {
 	RHSystemConfigData	*sys_data;
@@ -661,6 +698,38 @@ void nm_system_deactivate_all_dialup (GSList *list)
 	}
 }
 
+
+gboolean nm_system_deactivate_dialup (GSList *list, const char *dialup)
+{
+	GSList *l;
+	gboolean ret = FALSE;
+	
+	for (l = list; l; l = g_slist_next (l))
+	{
+		NMDialUpConfig *config = (NMDialUpConfig *) l->data;
+		if (strcmp (dialup, config->name) == 0)
+		{
+			char *cmd;
+			int status;
+			
+			nm_info ("Dectivating dialup device %s (%s) ...", dialup, (char *) config->data);
+			cmd = g_strdup_printf ("/sbin/ifdown %s", (char *) config->data);
+			status = nm_spawn_process (cmd);
+			g_free (cmd);
+			if (status == 0) {
+				ret = TRUE;
+			} else {
+				/* FIXME: Decode errors into something sensible */
+				nm_warning ("Couldn't deactivate dialup device %s (%s) - %d", dialup, (char *) config->data, status);
+				ret = FALSE;
+			}
+			break;
+		}
+	}
+	return ret;
+}
+
+
 gboolean nm_system_activate_dialup (GSList *list, const char *dialup)
 {
 	GSList *l;
@@ -820,3 +889,44 @@ GSList * nm_system_get_dialup_config (void)
 	
 	return list;
 }
+
+/*
+ * nm_system_activate_nis
+ *
+ * set up the nis domain and write a yp.conf
+ *
+ */
+void nm_system_activate_nis (NMIP4Config *config)
+{
+}
+
+/*
+ * nm_system_shutdown_nis
+ *
+ * shutdown ypbind
+ *
+ */
+void nm_system_shutdown_nis (void)
+{
+}
+
+/*
+ * nm_system_set_hostname
+ *
+ * set the hostname
+ *
+ */
+void nm_system_set_hostname (NMIP4Config *config)
+{
+}
+
+
+/*
+ * nm_system_should_modify_resolv_conf
+ *
+ * Can NM update resolv.conf, or is it locked down?
+ */
+gboolean nm_system_should_modify_resolv_conf (void)
+{
+	return TRUE;
+}
